/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;

import java.util.ArrayList;
/**
 * Classe qui permet de calculer la valeur propre
 * @author jama
 *
 */
public class EigenVector 
{
	/** Row and column dimension (square matrix).
	   @serial matrix dimension.
	 */
	private int n;
	/** Symmetry flag.
	   @serial internal symmetry flag.
	 */
	private boolean issymmetric;
	/** Arrays for internal storage of eigenvalues.
	   @serial internal storage of eigenvalues.
	 */
	private ArrayList<Double> d, e;
	/** Array for internal storage of eigenvectors.
	   @serial internal storage of eigenvectors.
	 */
	private Matrice<Double> V;
	/** Array for internal storage of nonsymmetric Hessenberg form.
	   @serial internal storage of nonsymmetric Hessenberg form.
	 */
	private Matrice<Double> H;
	/** Working storage for nonsymmetric algorithm.
	   @serial working storage for nonsymmetric algorithm.
	 */
	private ArrayList<Double> ort;
	private Matrice<Double> A;

	public EigenVector(Matrice<Double> m)
	{
		A = m;
		n = m.getNbLigne();
		V = new Matrice<Double>(n,n);
		d = initArrayList(n);
		e = initArrayList(n);
		

		issymmetric = true;
		for (int j = 0; (j < n) & issymmetric; j++) {
			for (int i = 0; (i < n) & issymmetric; i++) {
				issymmetric = (A.getCase(i, j).equals(A.getCase(j, i)));
			}
		}
		if (issymmetric) 
		{
			for (int i=0; i<n; i++) 
				for (int j=0; j<n; j++)
					V.setCase(i, j,new Double(A.getCase(i, j)));

			// Tridiagonalize.
			tred2();

			// Diagonalize.
			tql2();
		}
		else
		{
			
			H = new Matrice<Double>(n,n);
			ort = initArrayList(n);

			for (int j = 0; j < n; j++) {
				for (int i = 0; i < n; i++) {
					H.setCase(i, j, A.getCase(i, j));
				}
			}

			// Reduce to Hessenberg form.
			orthes();

			// Reduce Hessenberg to real Schur form.
			hqr2();
		}
	}
	
	public Matrice<Double> getEigenVector()
	{
		return V;
	}

	public ArrayList<Double> initArrayList(int n)
	{
		ArrayList<Double> l = new ArrayList<Double>(n);
		for(int i=0; i<n; i++)
			l.add(0.0);
		return l;
	}
	
	public ArrayList<Double> getRealEigenvalues () {
	      return d;
	   }

	   /** Return the imaginary parts of the eigenvalues
	   @return     imag(diag(D))
	   */

	   public ArrayList<Double> getImagEigenvalues () {
	      return e;
	   }

	   /** Return the block diagonal eigenvalue matrix
	   @return     D
	   */

	   public Matrice getD () {
	      @SuppressWarnings("unused")
		Matrice<Double> X = new Matrice<Double>(n,n);
	      Matrice<Double> D = new Matrice<Double>(n,n);
	      for (int i = 0; i < n; i++) {
	         for (int j = 0; j < n; j++) {
	            D.setCase(i,j,0.0);
	         }
	         D.setCase(i,i,d.get(i));
	         if (e.get(i) > 0) {
	            D.setCase(i, i+1, e.get(i));
	         } else if (e.get(i) < 0) {
	            D.setCase(i,i-1,e.get(i));
	         }
	      }
	      return D;
	   }


	private void tred2 () 
	{
		//  This is derived from the Algol procedures tred2 by
		//  Bowdler, Martin, Reinsch, and Wilkinson, Handbook for
		//  Auto. Comp., Vol.ii-Linear Algebra, and the corresponding
		//  Fortran subroutine in EISPACK.

		for (int j = 0; j < n; j++) {
			d.set(j, V.getCase(n-1, j));
		}
		// Householder reduction to tridiagonal form.

		for (int i=n-1; i>0; i--) 
		{
			// Scale to avoid under/overflow.
			double scale = 0.0;
			double h = 0.0;
			for (int k = 0; k < i; k++) 
			{
				scale = scale + Math.abs(d.get(k));
			}

			if (scale == 0.0) 
			{
				e.set(i, d.get(i-1));
				for (int j = 0; j < i; j++) 
				{
					d.set(j, V.getCase(i-1, j));
					V.setCase(i, j, 0.0);
					V.setCase(j, i, 0.0);
				}
			} 
			else 
			{
				// Generate Householder vector.
				for (int k = 0; k < i; k++) {
					d.set(k, d.get(k)/scale);
					h += d.get(k) * d.get(k);
				}
				double f = d.get(i-1);
				double g = Math.sqrt(h);
				if (f > 0) 
					g = -g;
				e.set(i, scale*g);
				h = h - f * g;
				d.set(i-1, f-g);
				for (int j = 0; j < i; j++) 
					e.set(j, 0.0);

				// Apply similarity transformation to remaining columns.
				for (int j = 0; j < i; j++) 
				{
					f = d.get(j);
					V.setCase(j, i, f);
					g = e.get(j) + V.getCase(j, j) * f;
					for (int k = j+1; k <= i-1; k++) 
					{
						g += V.getCase(k, j) * d.get(k);
						e.set(k, e.get(k) + V.getCase(k, j) * f);
					}
					e.set(j, g);
				}
				f = 0.0;
				for (int j = 0; j < i; j++) 
				{
					e.set(j, e.get(j) / h);
					f += e.get(j) * d.get(j);
				}
				double hh = f / (h + h);
				for (int j = 0; j < i; j++)
					e.set(j, e.get(j) - hh*d.get(j));

				for (int j = 0; j < i; j++)
				{
					f = d.get(j);
					g = e.get(j);
					for (int k = j; k <= i-1; k++) 
						V.setCase(k, j, V.getCase(k, j) - (f*e.get(k) + g*d.get(k)));

					d.set(j, V.getCase(i-1, j));
					V.setCase(i, j, 0.0);
				}
			}
			d.set(i, h);
		}

		// Accumulate transformations.
		for (int i = 0; i < n-1; i++) 
		{
			V.setCase(n-1, i, V.getCase(i, i));
			V.setCase(i, i, 1.0);
			double h = d.get(i+1);
			if (h != 0.0) 
			{
				for (int k = 0; k <= i; k++) 
					d.set(k, V.getCase(k, i+1) / h);
				for (int j = 0; j <= i; j++) 
				{
					double g = 0.0;
					for (int k = 0; k <= i; k++) 
						g = V.getCase(k, i+1) * V.getCase(k, j);
					for (int k = 0; k <= i; k++) 
						V.setCase(k, j, V.getCase(k, j) - g*d.get(k));
				}
			}

			for (int k = 0; k <= i; k++) 
				V.setCase(k, i+1, 0.0);
		}

		for (int j = 0; j < n; j++) 
		{
			d.set(j, V.getCase(n-1, j));
			V.setCase(n-1, j, 0.0);
		}
		V.setCase(n-1, n-1, 1.0);
		e.set(0, 0.0);
	} 


	// Symmetric tridiagonal QL algorithm.
	private void tql2 () 
	{
		//  This is derived from the Algol procedures tql2, by
		//  Bowdler, Martin, Reinsch, and Wilkinson, Handbook for
		//  Auto. Comp., Vol.ii-Linear Algebra, and the corresponding
		//  Fortran subroutine in EISPACK.

		for (int i=1; i<n; i++) 
			e.set(i-1, e.get(i));
		e.set(n-1, 0.0);

		double f = 0.0;
		double tst1 = 0.0;
		double eps = Math.pow(2.0,-52.0);
		for (int l = 0; l < n; l++) {

			// Find small subdiagonal element

			tst1 = Math.max(tst1,Math.abs(d.get(l)) + Math.abs(e.get(l)));
			int m = l;
			while (m < n) {
				if (Math.abs(e.get(m)) <= eps*tst1) {
					break;
				}
				m++;
			}

			// If m == l, d[l] is an eigenvalue,
			// otherwise, iterate.

			if (m > l) {
				int iter = 0;
				do {
					iter = iter + 1;  // (Could check iteration count here.)

					// Compute implicit shift

					double g = d.get(l);
					double p = (d.get(l+1) - g) / (2.0 * e.get(l));
					double r = hypot(p,1.0);
					if (p < 0)
						r = -r;
					d.set(l, e.get(l) / (p+r));
					d.set(l+1, e.get(l) * (p+r));
					double dl1 = d.get(l+1);
					double h = g - d.get(l);
					for (int i = l+2; i < n; i++)
						d.set(i, d.get(i) - h);
					f = f + h;

					// Implicit QL transformation.
					p = d.get(m);
					double c = 1.0;
					double c2 = c;
					double c3 = c;
					double el1 = e.get(l+1);
					double s = 0.0;
					double s2 = 0.0;
					for (int i = m-1; i >= l; i--) 
					{
						c3 = c2;
						c2 = c;
						s2 = s;
						g = c * e.get(i);
						h = c * p;
						r = hypot(p,e.get(i));
						e.set(i+1, s*r);
						s = e.get(i) / r;
						c = p / r;
						p = c * d.get(i) - s * g;
						d.set(i+1, h + s * (c * g + s * d.get(i)));

						// Accumulate transformation.
						for (int k = 0; k < n; k++) 
						{
							h = V.getCase(k, i+1);
							V.setCase(k, i+1, s*V.getCase(k, i) + c*h);
							V.setCase(k, i, c*V.getCase(k, i) - s*h);
						}
					}
					p = -s * s2 * c3 * el1 * e.get(l) / dl1;
					e.set(l, s*p);
					d.set(l, c*p);

					// Check for convergence.
				} while (Math.abs(e.get(l)) > eps*tst1);
			}
			d.set(l, d.get(l) + f);
			e.set(l, 0.0);
		}

		// Sort eigenvalues and corresponding vectors.

		for (int i = 0; i < n-1; i++) 
		{
			int k = i;
			double p = d.get(i);
			for (int j = i+1; j < n; j++) 
			{
				if (d.get(j) < p) 
				{
					k = j;
					p = d.get(j);
				}
			}
			if (k != i) 
			{
				d.set(k, d.get(i));
				d.set(i, p);
				for (int j = 0; j < n; j++) 
				{
					p = V.getCase(j, i);
					V.setCase(j, i, V.getCase(j, k));
					V.setCase(j, k, p);
				}
			}
		}
	}

	private void orthes () {
		   
	      //  This is derived from the Algol procedures orthes and ortran,
	      //  by Martin and Wilkinson, Handbook for Auto. Comp.,
	      //  Vol.ii-Linear Algebra, and the corresponding
	      //  Fortran subroutines in EISPACK.
	   
	      int low = 0;
	      int high = n-1;
	   
	      for (int m = low+1; m <= high-1; m++) {
	   
	         // Scale column.
	   
	         double scale = 0.0;
	         for (int i = m; i <= high; i++) {
	            scale = scale + Math.abs(H.getCase(i,m-1));
	         }
	         if (scale != 0.0) {
	   
	            // Compute Householder transformation.
	   
	            double h = 0.0;
	            for (int i = high; i >= m; i--) {
	               ort.set(i, H.getCase(i,m-1)/scale);
	               h += ort.get(i) * ort.get(i);
	            }
	            double g = Math.sqrt(h);
	            if (ort.get(m) > 0) {
	               g = -g;
	            }
	            h = h - ort.get(m) * g;
	            ort.set(m,ort.get(m) - g);
	   
	            // Apply Householder similarity transformation
	            // H = (I-u*u'/h)*H*(I-u*u')/h)
	   
	            for (int j = m; j < n; j++) {
	               double f = 0.0;
	               for (int i = high; i >= m; i--) {
	                  f += ort.get(i)*H.getCase(i,j);
	               }
	               f = f/h;
	               for (int i = m; i <= high; i++) {
	                  H.setCase(i, j,H.getCase(i,j)-f*ort.get(i));
	               }
	           }
	   
	           for (int i = 0; i <= high; i++) {
	               double f = 0.0;
	               for (int j = high; j >= m; j--) {
	                  f += ort.get(j)*H.getCase(i, j);
	               }
	               f = f/h;
	               for (int j = m; j <= high; j++) {
	                  H.setCase(i,j,H.getCase(i,j)-f*ort.get(j));
	               }
	            }
	            ort.set(m,scale*ort.get(m));
	            H.setCase(m,m-1,scale*g);
	         }
	      }
	   
	      // Accumulate transformations (Algol's ortran).

	      for (int i = 0; i < n; i++) {
	         for (int j = 0; j < n; j++) {
	            V.setCase(i,j,(i == j ? 1.0 : 0.0));
	         }
	      }

	      for (int m = high-1; m >= low+1; m--) {
	         if (H.getCase(m,m-1) != 0.0) {
	            for (int i = m+1; i <= high; i++) {
	               ort.set(i,H.getCase(i,m-1));
	            }
	            for (int j = m; j <= high; j++) {
	               double g = 0.0;
	               for (int i = m; i <= high; i++) {
	                  g += ort.get(i) * V.getCase(i, j);
	               }
	               // Double division avoids possible underflow
	               g = (g / ort.get(m)) / H.getCase(m, m-1);
	               for (int i = m; i <= high; i++) {
	                  V.setCase(i, j, V.getCase(i,j)+g*ort.get(i));
	               }
	            }
	         }
	      }
	   }


	   // Complex scalar division.

	   private transient double cdivr, cdivi;
	   private void cdiv(double xr, double xi, double yr, double yi) {
	      double r,d;
	      if (Math.abs(yr) > Math.abs(yi)) {
	         r = yi/yr;
	         d = yr + r*yi;
	         cdivr = (xr + r*xi)/d;
	         cdivi = (xi - r*xr)/d;
	      } else {
	         r = yr/yi;
	         d = yi + r*yr;
	         cdivr = (r*xr + xi)/d;
	         cdivi = (r*xi - xr)/d;
	      }
	   }


	   // Nonsymmetric reduction from Hessenberg to real Schur form.

	   private void hqr2 () {
	   
	      //  This is derived from the Algol procedure hqr2,
	      //  by Martin and Wilkinson, Handbook for Auto. Comp.,
	      //  Vol.ii-Linear Algebra, and the corresponding
	      //  Fortran subroutine in EISPACK.
	   
	      // Initialize
	   
	      int nn = this.n;
	      int n = nn-1;
	      int low = 0;
	      int high = nn-1;
	      double eps = Math.pow(2.0,-52.0);
	      double exshift = 0.0;
	      double p=0,q=0,r=0,s=0,z=0,t,w,x,y;
	   
	      // Store roots isolated by balanc and compute matrix norm
	   
	      double norm = 0.0;
	      for (int i = 0; i < nn; i++) {
	         if (i < low | i > high) {
	            d.set(i, H.getCase(i, i));
	            e.set(i,0.0);
	         }
	         for (int j = Math.max(i-1,0); j < nn; j++) {
	            norm = norm + Math.abs(H.getCase(i,j));
	         }
	      }

	      // Outer loop over eigenvalue index
	   
	      int iter = 0;
	      while (n >= low) {
	   
	         // Look for single small sub-diagonal element
	   
	         int l = n;
	         while (l > low) {
	            s = Math.abs(H.getCase(l-1,l-1)) + Math.abs(H.getCase(l, l));
	            if (s == 0.0) {
	               s = norm;
	            }
	            if (Math.abs(H.getCase(l, l-1)) < eps * s) {
	               break;
	            }
	            l--;
	         }

	         // Check for convergence
	         // One root found
	   
	         if (l == n) {
	            H.setCase(n,n,H.getCase(n, n)+ exshift);
	            d.set(n, H.getCase(n,n));
	            e.set(n,0.0);
	            n--;
	            iter = 0;
	   
	         // Two roots found
	   
	         } else if (l == n-1) {
	            w = H.getCase(n, n-1)* H.getCase(n-1, n);
	            p = (H.getCase(n-1, n-1)- H.getCase(n, n)) / 2.0;
	            q = p * p + w;
	            z = Math.sqrt(Math.abs(q));
	            H.setCase(n,n,H.getCase(n, n)+ exshift);
	            H.setCase(n-1, n-1, H.getCase(n-1, n-1)+exshift);
	            x = H.getCase(n, n);
	   
	            // Real pair
	   
	            if (q >= 0) {
	               if (p >= 0) {
	                  z = p + z;
	               } else {
	                  z = p - z;
	               }
	               d.set(n-1, x+z);
	               d.set(n,d.get(n-1));
	               if (z != 0.0) {
	                  d.set(n, x - w / z);
	               }
	               e.set(n-1, 0.0);
	               e.set(n, 0.0);
	               x = H.getCase(n,n-1);
	               s = Math.abs(x) + Math.abs(z);
	               p = x / s;
	               q = z / s;
	               r = Math.sqrt(p * p+q * q);
	               p = p / r;
	               q = q / r;
	   
	               // Row modification
	   
	               for (int j = n-1; j < nn; j++) {
	                  z = H.getCase(n-1, j);
	                  H.setCase(n-1, j, q*z+p*H.getCase(n, j));
	                  H.setCase(n, j, q*H.getCase(n, j)-p*z);
	               }
	   
	               // Column modification
	   
	               for (int i = 0; i <= n; i++) {
	                  z = H.getCase(i, n-1);
	                  H.setCase(i, n-1, q*z+p*H.getCase(i, n));
	                  H.setCase(i, n, q*H.getCase(i, n)-p*z);
	               }
	   
	               // Accumulate transformations
	   
	               for (int i = low; i <= high; i++) {
	                  z = V.getCase(i,n-1);
	                  V.setCase(i, n-1, q * z + p * V.getCase(i,n));
	                  V.setCase(i, n, q*V.getCase(i,n)-p*z);
	               }
	   
	            // Complex pair
	   
	            } else {

	               d.set((n-1), x + p);
	               d.set(n,x + p);
	               e.set(n-1, z);
	               e.set(n, -z);
	            }
	            n = n - 2;
	            iter = 0;
	   
	         // No convergence yet
	   
	         } else {
	   
	            // Form shift
	   
	            x = H.getCase(n,n);
	            y = 0.0;
	            w = 0.0;
	            if (l < n) {
	               y = H.getCase(n-1,n-1);
	               w = H.getCase(n,n-1)* H.getCase(n-1,n);
	            }
	   
	            // Wilkinson's original ad hoc shift
	   
	            if (iter == 10) {
	               exshift += x;
	               for (int i = low; i <= n; i++) {
	                  H.setCase(i,i,H.getCase(i,i)-x);
	               }
	               s = Math.abs(H.getCase(n, n-1)) + Math.abs(H.getCase(n-1, n-2));
	               x = y = 0.75 * s;
	               w = -0.4375 * s * s;
	            }

	            // MATLAB's new ad hoc shift

	            if (iter == 30) {
	                s = (y - x) / 2.0;
	                s = s * s + w;
	                if (s > 0) {
	                    s = Math.sqrt(s);
	                    if (y < x) {
	                       s = -s;
	                    }
	                    s = x - w / ((y - x) / 2.0 + s);
	                    for (int i = low; i <= n; i++) {
	                       H.setCase(i,i,H.getCase(i,i)-s);
	                    }
	                    exshift += s;
	                    x = y = w = 0.964;
	                }
	            }
	   
	            iter = iter + 1;   // (Could check iteration count here.)
	   
	            // Look for two consecutive small sub-diagonal elements
	   
	            int m = n-2;
	            while (m >= l) {
	               z = H.getCase(m, m);
	               r = x - z;
	               s = y - z;
	               p = (r * s - w) / H.getCase(m+1, m) + H.getCase(m,m+1);
	               q = H.getCase(m+1,m+1) - z - r - s;
	               r = H.getCase(m+2, m+1);
	               s = Math.abs(p) + Math.abs(q) + Math.abs(r);
	               p = p / s;
	               q = q / s;
	               r = r / s;
	               if (m == l) {
	                  break;
	               }
	               if (Math.abs(H.getCase(m,m-1)) * (Math.abs(q) + Math.abs(r)) < eps * (Math.abs(p) * (Math.abs(H.getCase(m-1,m-1)) + Math.abs(z) + Math.abs(H.getCase(m+1, m+1))))) 
	               {
	                     break;
	               }
	               m--;
	            }
	   
	            for (int i = m+2; i <= n; i++) {
	               H.setCase(i,i-2, 0.0);
	               if (i > m+2) {
	                  H.setCase(i, i-3, 0.0);
	               }
	            }
	   
	            // Double QR step involving rows l:n and columns m:n
	   
	            for (int k = m; k <= n-1; k++) {
	               boolean notlast = (k != n-1);
	               if (k != m) {
	                  p = H.getCase(k,k-1);
	                  q = H.getCase(k+1,k-1);
	                  r = (notlast ? H.getCase(k+2,k-1) : 0.0);
	                  x = Math.abs(p) + Math.abs(q) + Math.abs(r);
	                  if (x != 0.0) {
	                     p = p / x;
	                     q = q / x;
	                     r = r / x;
	                  }
	               }
	               if (x == 0.0) {
	                  break;
	               }
	               s = Math.sqrt(p * p + q * q + r * r);
	               if (p < 0) {
	                  s = -s;
	               }
	               if (s != 0) {
	                  if (k != m) {
	                     H.setCase(k,k-1,-s*x);
	                  } else if (l != m) {
	                     H.setCase(k,k-1,-H.getCase(k, k-1));
	                  }
	                  p = p + s;
	                  x = p / s;
	                  y = q / s;
	                  z = r / s;
	                  q = q / p;
	                  r = r / p;
	   
	                  // Row modification
	   
	                  for (int j = k; j < nn; j++) {
	                     p = H.getCase(k, j) + q * H.getCase(k+1,j);
	                     if (notlast) {
	                        p = p + r * H.getCase(k+2, j);
	                        H.setCase(k+2,j,H.getCase(k+2,j)-p*z);
	                     }
	                     H.setCase(k,j,H.getCase(k,j)-p*x);
	                     H.setCase(k+1,j,H.getCase(k+1,j)-p*y);
	                  }
	   
	                  // Column modification
	   
	                  for (int i = 0; i <= Math.min(n,k+3); i++) {
	                     p = x * H.getCase(i,k) + y * H.getCase(i, k+1);
	                     if (notlast) {
	                        p = p + z * H.getCase(i, k+2);
	                        H.setCase(i, k+2, H.getCase(i, k+2)-p*r);
	                     }
	                     H.setCase(i, k, H.getCase(i,k)-p);
	                     H.setCase(i, k+1, H.getCase(i,k+1)-p*q);
	                  }
	   
	                  // Accumulate transformations
	   
	                  for (int i = low; i <= high; i++) {
	                     p = x * V.getCase(i, k) + y * V.getCase(i, k+1);
	                     if (notlast) {
	                        p = p + z * V.getCase(i,k+2);
	                        V.setCase(i,k+2,V.getCase(i,k+2)-p*r);
	                     }
	                     V.setCase(i,k,V.getCase(i, k)-p);
	                     V.setCase(i, k+1, V.getCase(i, k+1)-p*q);
	                  }
	               }  // (s != 0)
	            }  // k loop
	         }  // check convergence
	      }  // while (n >= low)

		
	      // Backsubstitute to find vectors of upper triangular form

	      if (norm == 0.0) {
	         return;
	      }
	   
	      for (n = nn-1; n >= 0; n--) {
	         p = d.get(n);
	         q = e.get(n);	   
	         // Real vector
	   
	         if (q == 0) {
	            int l = n;
	            H.setCase(n, n, 1.0);
	            for (int i = n-1; i >= 0; i--) {
	               w = H.getCase(i, i) - p;
	               r = 0.0;
	               for (int j = l; j <= n; j++) {
	                  r = r + H.getCase(i,j)* H.getCase(j, n);
	               }
	               if (e.get(i) < 0.0) {
	                  z = w;
	                  s = r;
	               } else {
	                  l = i;
	                  if (e.get(i) == 0.0) {
	                     if (w != 0.0) {
	                        H.setCase(i, n, -r/w);
	                     } else {
	                        H.setCase(i, n, -r/(eps*norm));
	                     }
	   
	                  // Solve real equations
	   
	                  } else {
	                     x = H.getCase(i, i+1);
	                     y = H.getCase(i+1,i);
	                     q = (d.get(i) - p) * (d.get(i) - p) + e.get(i) * e.get(i);
	                     t = (x * s - z * r) / q;
	                     H.setCase(i,n,t);
	                     if (Math.abs(x) > Math.abs(z)) {
	                        H.setCase(i+1,n,(-r - w * t) / x);
	                     } else {
	                        H.setCase(i+1,n,(-s - y * t) / z);
	                     }
	                  }
	   
	                  // Overflow control
	   
	                  t = Math.abs(H.getCase(i, n));
	                  if ((eps * t) * t > 1) {
	                     for (int j = i; j <= n; j++) {
	                        H.setCase(j,n,H.getCase(j,n)/ t);
	                     }
	                  }
	               }
	            }
	   
	         // Complex vector
	   
	         } else if (q < 0) {
	            int l = n-1;

	            // Last vector component imaginary so matrix is triangular
	   
	            if (Math.abs(H.getCase(n, n-1)) > Math.abs(H.getCase(n-1,n))) {
	               H.setCase(n-1,n-1,q / H.getCase(n,n-1));
	               H.setCase(n-1, n, -(H.getCase(n,n)-p)/H.getCase(n, n-1));
	            } else {
	               cdiv(0.0,-H.getCase(n-1, n),H.getCase(n-1, n-1)-p,q);
	               H.setCase(n-1, n-1,cdivr);
	               H.setCase(n-1,n,cdivi);
	            }
	            H.setCase(n,n-1,0.0);
	            H.setCase(n, n, 1.0);
	            for (int i = n-2; i >= 0; i--) {
	               double ra,sa,vr,vi;
	               ra = 0.0;
	               sa = 0.0;
	               for (int j = l; j <= n; j++) {
	                  ra = ra + H.getCase(i, j)* H.getCase(j, n-1);
	                  sa = sa + H.getCase(i, j)* H.getCase(j,n);
	               }
	               w = H.getCase(i,i)- p;
	   
	               if (e.get(i) < 0.0) {
	                  z = w;
	                  r = ra;
	                  s = sa;
	               } else {
	                  l = i;
	                  if (e.get(i) == 0) {
	                     cdiv(-ra,-sa,w,q);
	                     H.setCase(i,n-1,cdivr);
	                     H.setCase(i,n,cdivi);
	                  } else {
	   
	                     // Solve complex equations
	   
	                     x = H.getCase(i,i+1);
	                     y = H.getCase(i+1,i);
	                     vr = (d.get(i) - p) * (d.get(i) - p) + e.get(i) * e.get(i)- q * q;
	                     vi = (d.get(i) - p) * 2.0 * q;
	                     if (vr == 0.0 & vi == 0.0) {
	                        vr = eps * norm * (Math.abs(w) + Math.abs(q) +
	                        Math.abs(x) + Math.abs(y) + Math.abs(z));
	                     }
	                     cdiv(x*r-z*ra+q*sa,x*s-z*sa-q*ra,vr,vi);
	                     H.setCase(i,n-1, cdivr);
	                     H.setCase(i,n,cdivi);
	                     if (Math.abs(x) > (Math.abs(z) + Math.abs(q))) {
	                        H.setCase(i+1,n-1,(-ra - w * H.getCase(i, n-1) + q * H.getCase(i,n)) / x);
	                        H.setCase(i+1,n,(-sa - w * H.getCase(i,n)- q * H.getCase(i,n-1)) / x);
	                     } else {
	                        cdiv(-r-y*H.getCase(i,n-1),-s-y*H.getCase(i,n),z,q);
	                        H.setCase(i+1,n-1,cdivr);
	                        H.setCase(i+1,n,cdivi);
	                     }
	                  }
	   
	                  // Overflow control

	                  t = Math.max(Math.abs(H.getCase(i,n-1)),Math.abs(H.getCase(i,n)));
	                  if ((eps * t) * t > 1) {
	                     for (int j = i; j <= n; j++) {
	                        H.setCase(j,n-1,H.getCase(j,n-1) / t);
	                        H.setCase(j,n,H.getCase(j, n) / t);
	                     }
	                  }
	               }
	            }
	         }
	      }
	   
	      // Vectors of isolated roots
	   
	      for (int i = 0; i < nn; i++) {
	         if (i < low | i > high) {
	            for (int j = i; j < nn; j++) {
	               V.setCase(i, j, H.getCase(i,j));
	            }
	         }
	      }

         	
	      // Back transformation to get eigenvectors of original matrix
	   
	      for (int j = nn-1; j >= low; j--) {
	         for (int i = low; i <= high; i++) {
	            z = 0.0;
	            for (int k = low; k <= Math.min(j,high); k++) {
	               z = z + V.getCase(i,k) * H.getCase(k, j);
	            }
	            V.setCase(i,j,z);
	         }
	      }
	   }


	/** sqrt(a^2 + b^2) without under/overflow. **/
	public static double hypot(double a, double b) {
		double r;
		if (Math.abs(a) > Math.abs(b)) {
			r = b/a;
			r = Math.abs(a)*Math.sqrt(1+r*r);
		} else if (b != 0) {
			r = a/b;
			r = Math.abs(b)*Math.sqrt(1+r*r);
		} else {
			r = 0.0;
		}
		return r;
	}

}
