/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.dessin;

import graphe.Graphe;
import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.MenuTraitement;
import ihm.chemin.Chemin;
import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;
import ihm.transition.Transition;
import java.awt.*;
import java.awt.event.*;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.util.*;
import javax.swing.*;

import calcul.CalculCoutMultiplicatif;
import calcul.Matrice;
import calcul.MethodeResolution;
import calcul.UsingMarks;
import org.sbml.libsbml.KineticLaw;
import org.sbml.libsbml.ListOf;
import org.sbml.libsbml.Model;
import org.sbml.libsbml.Parameter;
import org.sbml.libsbml.Reaction;
import org.sbml.libsbml.SBMLDocument;
import org.sbml.libsbml.SBMLReader;
import org.sbml.libsbml.SimpleSpeciesReference;
import org.sbml.libsbml.Species;
import org.sbml.libsbml.SpeciesReference;

import thread.StartRechercheLocale;
import thread.ThreadCalcul;
import util.ConvertArrayToList;
import cout.Cout;
import dialog.ErreurDialog;

/**
 * Classe qui herite de la classe JPanel, qui implemente les interfaces
 * Serializable pour la sauvegarde et l'enregistrement
 * MouseListener et MouseMotionListener pour detecter les actions de la souris
 * et qui affiche le dessin du graphe, et qui contient les methodes de resolution
 * @author docet Giraudet
 */

public class DessinGraphe extends JPanel implements Serializable,MouseListener,MouseMotionListener
{
	static final long serialVersionUID = 0;
	public static Double EPSILON = new Double(Application.config.getProperty("epsilon"));
	protected transient MenuTraitement menuTraitement;
	protected transient EtatGraphique etat_selectionne;
	public Graphe graphe;
	protected transient Transition transition_selectionne;
	private RectangleDessin rect;
	private transient boolean point_selectionne;
	private transient DessinGrapheState state;
	public transient boolean resGraphe;
	public transient boolean resFile;
	private transient String pathFile;
	public ArrayList<MethodeResolution> methodes;
	public transient ArrayList<ThreadCalcul> threadCalculs; 

	static
	{
		System.loadLibrary("sbmlj");
	}
	
	/**
	 * Constructeur de la classe qui initialise les differentes listes
	 *
	 */
	public DessinGraphe()
	{
		super(null);
		setBorder(BorderFactory.createBevelBorder(1));
		setBackground(Color.WHITE);	
		graphe = new Graphe();
		rect=new RectangleDessin(0,0,0,0);
		point_selectionne=false;
		EtatGraphique.etatSuivant=1;
		addMouseListener(this);

		addMouseMotionListener(this);

		state = new DessinGrapheStateDefault(this);
		methodes = new ArrayList<MethodeResolution>();
		methodes.add(new UsingMarks(graphe));
		methodes.add(new CalculCoutMultiplicatif(graphe));
		threadCalculs = new ArrayList<ThreadCalcul>();
	}

	public void setGraphe(Graphe g)
	{
		graphe=g;
		methodes.set(0,new UsingMarks(graphe));
		methodes.set(1, new CalculCoutMultiplicatif(graphe));
	}
	
	public static void setEpsilon(Double eps)
	{
		EPSILON=eps;
	}
	

	@Override
	public Dimension getMinimumSize()
	{
		return new Dimension(this.getWidth(),this.getHeight());
	}

	@Override
	public Dimension getPreferredSize()
	{
		return getMinimumSize();
	}

	@Override
	public Dimension getMaximumSize()
	{
		return getMinimumSize();
	}


	public ArrayList<Cout> getListeCout()
	{
		ArrayList<Cout> res = new ArrayList<Cout>();
		res.addAll(graphe.getlisteMatriceCoutP1());
		return res;
	}

	public Transition getTransitionSelectionne()
	{
		return transition_selectionne;
	}

	public void setTransitionSelectionne(Transition t)
	{
		transition_selectionne=t;
	}

	/**
	 * Methode qui permet de rechercher un etat a partir de son numero
	 * @param s
	 * @return EtatGraphique
	 */
	public EtatGraphique rechercheEtatNum(int n)
	{
		return graphe.rechercheEtatNum(n);
	}

	public void setRayon(int t)
	{
		graphe.setRayon(t);
		repaint();
	}

	public void setResultatGraphe(boolean x)
	{
		resGraphe=x;
	}
	public void setResultatFile(boolean b)
	{
		resFile = b;
	}
	public void setPathFile(String s)
	{
		pathFile = s;
	}

	public void setColorEtat(Color c)
	{
		graphe.setColorEtat(c);
		repaint();
	}

	public void setColorTransition(Color c)
	{
		graphe.setColorTransition(c);
		repaint();
	}
	
	public boolean nomDifferentMethode(String nom)
	{
		for(int i=0;i<methodes.size();i++)
		{
			if(methodes.get(i).toString().compareTo(nom)==0)
				return false;
		}
		return true;
	}

	/**
	 * Methode qui permet de charger un fichier xml respectant la norme xml
	 * @param f
	 */
	public boolean ChargerSBMLFile(File f)
	{
		SBMLReader sr = new SBMLReader();
		SBMLDocument d = sr.readSBML(
				f.getPath());
		Model m = d.getModel();
		if(m!=null)
		{
			ListOf los = m.getListOfSpecies();
			ListOf lor = m.getListOfReactions();
			ListOf loparam = m.getListOfParameters();

			//on recupere la liste des etats
			for (int i = 0; i < m.getNumSpecies(); i++) 
			{
				Species s = (Species)los.get(i);
				EtatGraphique eg = new EtatGraphique(s.getId());
				graphe.listeEtats.add(eg);
			};

			//booleen pour savoir si les etats ont leur position
			boolean posEtatConnue = false;
			//on recupere la liste des parametres
			for(int i=0; i<m.getNumParameters();i++)
			{
				Parameter p = (Parameter)loparam.get(i);
				String s = p.getId();
				if(s.charAt(0)=='x')
				{//le parametre est une abscisse
					//on recupere l'etiquette de l'etat
					String num = s.substring(1,s.length());
					//on recherche si elle correspond a un etat
					for(int j=0; j<graphe.getNombreEtats();j++)
					{
						EtatGraphique e = graphe.listeEtats.get(j);
						if(e.getEtiquette().compareTo(num)==0)
						{
							//on ajoute la coordonnee
							e.setX((int)p.getValue());
						}
					}
					posEtatConnue = true;
				}
				else
					if(s.charAt(0)=='y')
					{//le parametre est une ordonnee
						//on recupere l'etiquette de l'etat
						String num = s.substring(1,s.length());
						//on recherche l'etat
						for(int j=0; j<graphe.getNombreEtats();j++)
						{
							EtatGraphique e = graphe.listeEtats.get(j);
							if(e.getEtiquette().compareTo(num)==0)
							{//on  modifie l'ordonnee de l'etat
								e.setY((int)p.getValue());
							}
						}
					}
			}
			//on recupere la liste des transitions
			for (int i=0;i<m.getNumReactions();i++)
			{
				EtatGraphique eg;
				Reaction r = (Reaction)lor.get(i);
				//liste des etats de depart
				ListOf lor1 = r.getListOfReactants();
				//liste des etats d'arrivee
				ListOf lop = r.getListOfProducts();
				Transition fl = new Transition();
				//on verifie qu'il y a qu'un etat d'arrivee
				if(lor1.getNumItems()>1)
				{
					new ErreurDialog(Application.langue.getProperty("erreurDG1"));
				}
				//on verifie que l'etat existe bien
				eg=rechercheEtatParEtiquette(((SimpleSpeciesReference)lor1.get(0)).getSpecies());
				if(eg==null)
				{
					new ErreurDialog(Application.langue.getProperty("erreurDG2"));
				}
				else
				{
					fl.setEtatDepart(eg);
					//on verifie qu'il y a qu'un etat de depart
					eg.addTransitionsSortantes(fl);
					if(lop.getNumItems()>1)
					{
						new ErreurDialog(Application.langue.getProperty("erreurDG4"));
					}
					//on verifie que l'etat existe bien
					eg=rechercheEtatParEtiquette(((SimpleSpeciesReference)lop.get(0)).getSpecies());
					if(eg==null)
					{
						new ErreurDialog(Application.langue.getProperty("erreurDG5")); 
					}
					else
					{
						fl.setEtatArrivee(eg);
						eg.addTransitionsEntrantes(fl);
						//on test s'il y a un intervalle
						if(r.isSetKineticLaw())
						{
							KineticLaw kl = r.getKineticLaw();
							//on recupere la liste des parametres
							ListOf param = kl.getListOfParameters();
							for (int l=0;l<kl.getNumParameters();l++)
							{
								//on recupere les intervalles deja cree pour cette transition
								ArrayList<Intervalle> intervalles = fl.getIntervalles();
								Parameter p = (Parameter)param.get(l);
								String s = p.getId();
								if(s.substring(0,4).compareTo("imin")==0)
								{//on a la borne min d'un intervalle
									//on recupere le nom de l'intervalle
									String des = s.substring(4);
									int a=0; 
									boolean trouve=false;
									Intervalle inter = null;
									while ((a<intervalles.size())&&!trouve)
									{//on cherche si l'intervalle existe deja
										inter = intervalles.get(a);
										if(inter.getDescription().compareTo(des)==0)
											trouve=true;
										else
											a++;
									}
									if(trouve)
									{//on modifie la borne de l'intervalle
										inter.setMin((int)p.getValue());
									}
									else 
									{//on cree un nouvel intervalle
										inter = new Intervalle(des);
										inter.setMin((int)p.getValue());
										fl.addIntervalle(inter);
									}
								}
								else
									if(s.substring(0,4).compareTo("imax")==0)
									{//on est dans le cas d'une borne max
										//on recupere le nom de l'intervalle
										String des = s.substring(4);
										int a=0; 
										boolean trouve=false;
										Intervalle inter = null;
										while ((a<intervalles.size())&&!trouve)
										{//on recherche si l'intervalle a ete cree
											inter = intervalles.get(a);
											if(inter.getDescription().compareTo(des)==0)
												trouve=true;
											else
												a++;
										}
										if(trouve)
										{//on modifie la borne max de l'intervalle
											inter.setMax((int)p.getValue());
										}
										else 
										{//on cree un nouvel intervalle
											inter = new Intervalle(des);
											inter.setMax((int)p.getValue());
											fl.addIntervalle(inter);
										}
									}
							}
						}
						graphe.listeTransitions.add(fl);
					}
				}
			}
			if(!posEtatConnue)
				graphe.definePositionState();
			return true;
		}
		else
		{
			JOptionPane.showMessageDialog(FenetrePrincipale.getInstance(), Application.langue.getProperty("erreurDG3"), Application.langue.getProperty("erreur"), JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}
	/**
	 * Methode pour sauvegarder sous format sbml
	 * @param m
	 */
	public void SauvegarderSBMLFile(Model m)
	{
		ArrayList<EtatGraphique> liste_etat = graphe.listeEtats;
		//on enregistre chaque
		for(int i =0;i<liste_etat.size();i++)
		{

			EtatGraphique e= liste_etat.get(i);
			Species s = new Species(""+e.getEtiquette());
			s.setCompartment("c1");
			m.addSpecies(s);
			Parameter px = new Parameter("x"+e.getEtiquette());
			px.setValue(e.getX());
			Parameter py = new Parameter("y"+e.getEtiquette());
			py.setValue(e.getY());
			m.addParameter(px);
			m.addParameter(py);	
		}
		//on enregistre chaque transition
		for(int i=0;i<graphe.listeTransitions.size();i++)
		{
			Transition f = graphe.listeTransitions.get(i);
			Reaction r = new Reaction("r"+i);
			SpeciesReference srr = new SpeciesReference(""+f.getEtatDepart().getEtiquette());
			r.addReactant(srr);
			SpeciesReference srp = new SpeciesReference(""+f.getEtatArrivee().getEtiquette());
			r.addProduct(srp);
			ArrayList<Intervalle> intervalles = f.getIntervalles();
			KineticLaw kl = new KineticLaw();
			//on enregistre chaque intervalle
			for (int j=0;j<intervalles.size();j++)
			{
				Intervalle inter = intervalles.get(j);
				Parameter imin = new Parameter("imin"+inter.getDescription());
				imin.setValue(inter.getMin());
				kl.addParameter(imin);
				Parameter imax = new Parameter("imax"+inter.getDescription());
				imax.setValue(inter.getMax());
				kl.addParameter(imax);

			}
			r.setKineticLaw(kl);
			m.addReaction(r);
		}
	}

	/**
	 * Methode qui permet de rechercher un etat a partir de son etiquette
	 * @param eti
	 * @return EtatGraphique
	 */
	public EtatGraphique rechercheEtatParEtiquette(String eti)
	{
		return graphe.rechercheEtatParEtiquette(eti);
	}

	/**
	 * Methode qui permet d'enregistrer une transition
	 * @param f
	 */
	public void enregistre_Transition (Transition f)
	{
		graphe.enregistreTransition(f);
	}
	/**
	 * Methode qui permet de ouvrir un fichier de type gda
	 *
	 */
	public void ouverture_fichier()
	{
		this.setState(new DessinGrapheStateDefault(this));
		graphe.listeTransitions = new ArrayList<Transition>();
		for(int i=0;i<graphe.getNombreEtats();i++)
		{
			EtatGraphique e = graphe.listeEtats.get(i);
			ArrayList<Transition> lfe = e.getTransitionsEntrantes();
			for(int j=0;j<lfe.size();j++)
			{
				Transition f=lfe.get(j);
				this.enregistre_Transition(f);
				f.setEtatArrivee(e);
			}
			ArrayList<Transition> lfs = e.getTransitionsSortantes();
			for(int j=0;j<lfs.size();j++)
			{
				Transition f=lfs.get(j);
				this.enregistre_Transition(f);
				f.setEtatDepart(e);
			}
		}
		threadCalculs = new ArrayList<ThreadCalcul>();
		EtatGraphique.etatSuivant=graphe.getNombreEtats()+1;
		rect.setLarg(this.getWidth());
		rect.setHaut(this.getHeight());
		rect.setMinimumHaut(this.getHeight());
		rect.setMinimumLarg(this.getWidth());

	}

	@Override
	public void paintComponent(Graphics g)
	{
		//on dessine l'arriere plan
		g.setColor(Color.WHITE);
		adapteRectangle();
		rect.DessineRectangle(g);
		//on dessine chaque etat
		for (int i =0;i<graphe.getNombreEtats();i++)
		{
			((EtatGraphique)graphe.listeEtats.get(i)).dessineEtat(g);
		}
		//on dessine chaque fleche
		Matrice<Double> res=null;
		if(resGraphe)
			res =graphe.getMatriceResultat();
		for(int i=0;i<graphe.listeTransitions.size();i++)
		{
			Double d=null;
			Transition t=graphe.listeTransitions.get(i);
			if(res!=null)
			{
				int arr= graphe.listeEtats.indexOf(t.getEtatArrivee());
				int dep= graphe.listeEtats.indexOf(t.getEtatDepart());
				d = res.getCase(dep,arr);
			}
			t.dessineTransition(g,d);
		}
	}

	/**
	 * Methode qui permet d'acceder au boolean point_selectionne
	 * @return point_selectionne boolean
	 */
	public boolean getPointSelectionne()
	{
		return point_selectionne;
	}

	/**
	 * Methode qui permet d'acceder au rectangle de dessin
	 * @return rect RectangleDessin
	 */
	public RectangleDessin getRect()
	{
		return rect;
	}
	
	/**	 
	 * Methode qui retourne l'etat selectionne
	 * @return etat_selectionne EtatGraphique
	 */
	public EtatGraphique getEtatSelectionne()
	{
		return etat_selectionne;
	}

	/**
	 * Methode qui permet de modifier la variable etat_selectionne
	 * @param e
	 */
	public void setEtatSelectionne(EtatGraphique e)
	{
		etat_selectionne=e;
	}

	/**
	 * Methode qui permet de changer l'etat du dessin
	 * Par exemple pour aller dans l'etat ajout fleche
	 * setState(new DessinGrapheStateAddTransition())
	 * @param dgs
	 */
	public void setState(DessinGrapheState dgs)
	{
		state = dgs;
	}

	/**
	 * Methode qui permet de mettre a true ou false
	 * si le coin inferieur droit du rectangle est selectionne
	 * @param x
	 */
	public void setPointSelectionne(boolean x)
	{
		point_selectionne=x;
	}


	/**
	 * Methode qui permet d'ajouter une transition
	 * @param f
	 */
	public void addTransition(Transition f)
	{
		graphe.ajoutTransition(f);
	}

	/**
	 * Methode qui ajoute un etat au dessin
	 * @param j
	 */
	public void addetat(EtatGraphique j)
	{
		//on ajoute un etat
		graphe.ajoutEtat(j);
	}

	/**
	 * Methode pour savoir si on a clique sur le coin inferieur droit du rectangle de dessin
	 * @param x
	 * @param y
	 * @return boolean
	 */
	public boolean selectPoint(int x,int y)
	{
		return ((x>=(rect.getLarg()-5)) && (x<=rect.getLarg()) && (y<=rect.getHaut()) && (y>=(rect.getHaut()-5)));

	}

	/**
	 * Methode pour obtenir la liste de transition
	 * @return liste_Transition ArrayList<Transition>
	 */
	public ArrayList<Transition> getListeTransition()
	{
		return graphe.listeTransitions;
	}

	/**
	 * Fonction qui recherche un etat dans la liste en fonction de la position du clic
	 * @param x abcisse
	 * @param y ordonn
	 * @return etat clique, null sinon
	 */
	public EtatGraphique rechercheEtatGraphique(int x, int y)
	{
		return graphe.rechercheEtatGraphique(x, y);
	}

	/**
	 * Methode pour supprimer les transitions d'un etat
	 * @param etat
	 */
	public void removeTransitionEtat(EtatGraphique etat)
	{
		ArrayList<Transition> Transitions_entrantes = etat.getTransitionsEntrantes();
		ArrayList<Transition> Transitions_sortantes = etat.getTransitionsSortantes();
		for(int i=0;i<=Transitions_entrantes.size()-1;i++)
		{
			Transition t = Transitions_entrantes.get(i);
			t.getEtatDepart().removeTransitionsSortantes(t);
			graphe.listeTransitions.remove(t);
		}
		for(int i=0;i<=Transitions_sortantes.size()-1;i++)
		{
			Transition t = Transitions_sortantes.get(i);
			t.getEtatArrivee().removeTransitionsEntrantes(t);
			graphe.listeTransitions.remove(t);
		}
	}

	
	
	/**
	 * Fonction qui recherche une Transition dans la liste en fonction de la position du clic
	 * @param x abcisse
	 * @param y ordonn
	 * @return Transition cliqu, null sinon
	 */
	public Transition rechercheTransitionGraphique(int x, int y)
	{	
		return graphe.rechercheTransitionGraphique(x, y);		
	}


	/**
	 * Methode pour modifier le menu
	 * @param mt
	 */
	public void setMenu(MenuTraitement mt)
	{
		menuTraitement = mt;
	}

	/**
	 * Permet d'afficher les informations d'un état sélectionné dans le menu
	 */
	public void afficheInformationEtat(EtatGraphique eg)
	{
		JPanel pRes = new JPanel(); 
		pRes.setLayout(new BoxLayout(pRes,BoxLayout.Y_AXIS));
		JLabel lbnum = new JLabel(eg.toString());
		pRes.add(lbnum);
		menuTraitement.setConsultationEtat(pRes,eg);
	}
	/**
	 * Permet d'afficher les informations d'un état sélectionné dans le menu
	 */
	public void afficheInformationTransition(Transition f)
	{
		JPanel pRes = new JPanel(); 
		pRes.setLayout(new BoxLayout(pRes,BoxLayout.Y_AXIS));
		JLabel lbnum = new JLabel(f.toString());
		pRes.add(lbnum);
		JList liste  = new JList(ConvertArrayToList.convertStringList(f.getIntervalles()));
		liste.setSelectionMode(1);
		liste.setSelectedIndex(0);
		liste.setVisibleRowCount(3);
		JScrollPane defil = new JScrollPane(liste);

		pRes.add(defil);

		menuTraitement.setConsultationTransition(pRes,f);
	}

	/**
	 * Methode pour avoir la liste des chemins
	 * @return liste_chemin ArrayList<Chemin>
	 */
	public ArrayList<Chemin> getChemins()
	{
		return graphe.getChemins();
	}

	public Cout rechercheCoutNom(String nom)
	{
		return graphe.rechercheCout(nom);
	}

	public void SupprimeCout(Cout c)
	{
		graphe.SupprimeCout(c);
	}


	public void RechercheLocale(ArrayList<MethodeResolution> mr,JButton jb)
	{
		StartRechercheLocale srl = new StartRechercheLocale(graphe,mr,EPSILON,jb);
		threadCalculs.add(srl);
		menuTraitement.majThread();
		srl.start();
	}
	
/**
 * methode pour enregistrer une matrice resultat
 * @param m
 */
	public void createFileResult(Matrice<Double> m)
	{
		if(m!=null)
		{
		try {
			if(pathFile==null)
				pathFile = "result.csv";
			BufferedWriter fic = new BufferedWriter(new FileWriter(pathFile,true));
			fic.write(m.toCSV());
			fic.newLine();
			fic.newLine();
			fic.close();
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
		}
	}

	public void reorganiserGrappa()
	{
		graphe.reorganiserGrappa();
		//repaint();
	}
	
	/**
	 * methode pour adpater le rectangle de dessin au graphe
	 *
	 */
	public void adapteRectangle()
	{
		int maxX=0;
		int maxY=0;
		for(int i=0;i<graphe.getNombreEtats();i++)
		{
			EtatGraphique et=graphe.listeEtats.get(i);
			if(et.getX()>maxX)
				maxX=et.getX();
			if(et.getY()>maxY)
				maxY=et.getY();
		}
		if((rect.getLarg()==0 && rect.getHaut()==0)||(rect.getLarg()<maxX)||(rect.getHaut()<maxY))
		{
			
			if((maxX+2*graphe.getRayonMaxEtat())<this.getWidth())
				rect.setLarg(this.getWidth());
			else
				rect.setLarg(maxX+2*graphe.getRayonMaxEtat());
			if((maxY+2*graphe.getRayonMaxEtat())<this.getHeight())
				rect.setHaut(this.getHeight());
			else
				rect.setHaut(maxY+2*graphe.getRayonMaxEtat());
			if(maxX+2*graphe.getRayonMaxEtat()>this.getWidth())
				if(maxY+2*graphe.getRayonMaxEtat()>this.getHeight())
					this.setSize(maxX+2*graphe.getRayonMaxEtat(),maxY+2*graphe.getRayonMaxEtat());
				else
					this.setSize(maxX+2*graphe.getRayonMaxEtat(),this.getHeight());
			else
				if(maxY+2*graphe.getRayonMaxEtat()>this.getHeight())
					this.setSize(this.getWidth(),maxY+2*graphe.getRayonMaxEtat());
			rect.setMinimumHaut(this.getHeight());
			rect.setMinimumLarg(this.getWidth());
		}
		else
		{
			if(rect.getLarg()<this.getWidth())
			{
				rect.setLarg(this.getWidth());
				rect.setMinimumLarg(this.getWidth());
			}
			if(rect.getHaut()<this.getHeight())
			{
				rect.setHaut(this.getHeight());
				rect.setMinimumHaut(this.getHeight());
			}
		}
		
	}

	public void afficheChemin(Chemin c)
	{
		ArrayList<Transition> lt = c.getListeTransitions();
		for(int i=0;i<lt.size();i++)
			lt.get(i).setColorSel(Color.red);
		repaint();
	}

	public void mouseClicked(MouseEvent e) {
		state.mouseClicked(e);
	}

	public void mousePressed(MouseEvent e) {
		//on affiche les informations contenues sur l'etat ou la transition sur le lequel l'utilisateur a clique
		graphe.setColorDefaut();
		EtatGraphique etat=this.rechercheEtatGraphique(e.getX(),e.getY());
		if(etat!=null)
		{
			afficheInformationEtat(etat);
			etat.setColorSelection(Color.RED);
		}
		transition_selectionne = rechercheTransitionGraphique(e.getX(),e.getY());
		if(transition_selectionne!=null)
		{
			afficheInformationTransition(transition_selectionne);
			transition_selectionne.setColorSel(Color.RED);
		}
		if(etat == null && transition_selectionne == null)
		{
			menuTraitement.setConsultationVide();
		}
		repaint();
		state.mousePressed(e);
	}




	public void mouseReleased(MouseEvent e) {
		state.mouseReleased(e);
	}




	public void mouseEntered(MouseEvent e) {
		state.mouseEntered(e);	
	}




	public void mouseExited(MouseEvent e) {
		state.mouseExited(e);
	}	





	public void mouseDragged(MouseEvent e) {
		state.mouseDragged(e);
	}




	public void mouseMoved(MouseEvent e) {
		state.mouseMoved(e);	
	}



}
