/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.transition;

import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;
import java.awt.*;
import java.io.Serializable;
import java.util.*;
import javax.swing.*;

/**
 * Classe servant a representer les transitions du graphe
 * @author giraudet-e
 *
 */
public class Transition extends JPanel implements Serializable
{
	static final int ECART_SELECT = 5;
	static final double pi = Math.PI;
	static final double ANGLE_ECART = pi / 8;
	private static final long serialVersionUID = 1L;
	protected Point posDep;
	protected Point posArr;
	protected transient EtatGraphique eDepart;
	protected transient EtatGraphique eArrivee;
	
	protected RelationProba relation;
	private static Color c=Color.BLACK;
	protected Color csel;
	
	protected ArrayList<Intervalle> intervalles = new ArrayList<Intervalle>();
	
	public Transition()
	{
		posDep = new Point();
		posArr = new Point();
		eDepart=null;
		eArrivee=null;
		relation = null;
	}
	
	public Transition(EtatGraphique dep, EtatGraphique arr)
	{
		eDepart = dep;
		eArrivee = arr;
		disposeTransition();
	}
	
	public void setColor(Color col)
	{
		c=col;
	}
	
	public void setColorSel(Color c)
	{
		csel=c;
	}
	
	public void setDepart(int x, int y)
	{
		posDep.setLocation(x,y);
	}
	public void setArrivee(int x, int y)
	{
		posArr.setLocation(x,y);
	}
	
	public void setEtatDepart(EtatGraphique eg)
	{
		eDepart = eg;
	}
	public void setEtatArrivee(EtatGraphique eg)
	{
		eArrivee = eg;
	}
	
	public EtatGraphique getEtatDepart()
	{
		return eDepart;
	}
	public EtatGraphique getEtatArrivee()
	{
		return eArrivee;
	}
	public ArrayList<Intervalle> getIntervalles()
	{
		return intervalles;
	}
	public Intervalle getIntervalleProbabilite()
	{
		return intervalles.get(0);
	}
	public boolean estBoucle()
	{
		return eDepart.equals(eArrivee);
	}
	
	public void setRelation(RelationProba rp)
	{
		relation = rp;
		double val1 = rp.getAlpha()*rp.getTransitionP2().getIntervalleProbabilite().getMinProba();
		double val2 = rp.getAlpha()*rp.getTransitionP2().getIntervalleProbabilite().getMaxProba();
		intervalles.set(0, new Intervalle(Math.min(val1, val2)*100,Math.max(val1, val2)*100));
	}
	public RelationProba getRelation()
	{
		return relation;
	}
	
	
	/**
	 * Gestion des Intervalles
	 */
	public void addIntervalle(Intervalle iv)
	{
		intervalles.add(iv);
	}
	public void removeIntervalle(Intervalle iv)
	{
		intervalles.remove(iv);
	}	
	
	/**
	 * verifie si le point (x,y) appartient  une droite
	 */
	public boolean estSurTransition(int x, int y)
	{
		if(eDepart != null && eArrivee != null)
		{
			if(!eArrivee.equals(eDepart))
			{
				int x1 = eDepart.getCentre().x;
				int y1 = eDepart.getCentre().y;
				int x2 = eArrivee.getCentre().x;
				int y2 = eArrivee.getCentre().y;
				
				double teta = calculTeta(x1, y1, x2, y2);
				Point M1 = calculM1(teta, x1, y1);
				Point M2 = calculM2(teta, x2, y2);
				
				//if(eDepart.getnum()==3 && eArrivee.getnum()==5)
					//System.out.println("M1="+M1.x + " M2="+M2.x);
				if(Math.abs(M1.x-M2.x)<5)
				{
					return ( x<M1.x+ECART_SELECT && x>M1.x-ECART_SELECT && y>Math.min(M2.y,M1.y) && y<Math.max(M2.y,M1.y));
				}
				else
				{
					double alpha = getEquationA(M1, M2);
					int tmp = (int)(alpha*x + getEquationB(M1, M2));
					return ( y<tmp+ECART_SELECT && y>tmp-ECART_SELECT && x>Math.min(M2.x,M1.x) && x<Math.max(M2.x,M1.x));
				}
			}
			else
			{
				return (x<eDepart.getX()+10) && (x>eDepart.getX()-10) && (y<eDepart.getY()+10) && (y>eDepart.getY()-10);
			}
		}
		else
			return false;
	}
	
	
	/**
	 * Calcul equation de droite entre les points M1 er M2
	 */
	public static double getEquationA(Point M1, Point M2)
	{
		double x1 = M1.x;
		double y1 = M1.y;
		double x2 = M2.x;
		double y2 = M2.y;
		return ((y1-y2)/(x1-x2));
	}
	public static double getEquationB(Point M1, Point M2)
	{
		return M2.y - getEquationA(M1,M2)*M2.x;
	}
	
	
	/**
	 * Permet de placer la Transition entre des tats
	 */
	public void disposeTransition()
	{
		posDep = new Point(eDepart.getCentre());
		posArr = new Point(eArrivee.getCentre());
	}
	
	
	/**
	 * Permet de dessiner une Transition
	 */
	public void dessineTransition(final Graphics g,Double _resultat)
	{
		super.paintComponent(g);
		if(csel==null)
			g.setColor(c);
		else
			g.setColor(csel);
		if (eDepart.equals(eArrivee))
			new Boucle(eDepart.getCentre(),new Point(eDepart.getX(),eDepart.getY()),eDepart.getRayon()).dessineBoucle(g,_resultat);
		else
			dessineFleche(g,_resultat);
			
			
	}
	
	public double calculTeta(int x1, int y1, int x2, int y2)
	{
		double hypothenuse =  Math.sqrt( Math.pow(x2-x1,2) + Math.pow(y2-y1,2) );
		double teta = Math.acos( (x2 - x1) / hypothenuse );
		if(y2>y1)
			teta = pi*2-teta;
		return teta;
	}
	
	public Point calculM1(double teta, int x1, int y1)
	{
		Point M1 = new Point();
		M1.x = (int) (x1 + eDepart.getRayon() * Math.cos(teta + ANGLE_ECART));
		M1.y = (int) (y1 - eDepart.getRayon() * Math.sin(teta + ANGLE_ECART));
		return M1;
	}
	
	public Point calculM2(double teta, int x2, int y2)
	{
		Point M2 = new Point();
		M2.x = (int) (x2 - eArrivee.getRayon() * Math.cos(teta - ANGLE_ECART));
		M2.y = (int) (y2 + eArrivee.getRayon() * Math.sin(teta - ANGLE_ECART));
		return M2;
	}
	
	public void dessineFleche(Graphics g, Double res)
	{
		Point M1 = new Point();
		Point M2 = new Point();
		int x1,x2,y1,y2;	
		
		x1 = (int)eDepart.getCentre().getX();
		y1 = (int)eDepart.getCentre().getY();
		
		if(eArrivee==null)
		{
			x2 = posArr.x;
			y2 = posArr.y;
		}
		else
		{
			x2 = (int)eArrivee.getCentre().getX();
			y2 = (int)eArrivee.getCentre().getY();
		}
		
		double teta = calculTeta(x1, y1, x2, y2);
		
		M1 = calculM1(teta, x1, y1);
		
		if(eArrivee!=null)
		{
			M2 = calculM2(teta, x2, y2);
			new Fleche(M1,M2).dessineFleche(g);
		}
		else
		{
			new Fleche(M1.x,M1.y,x2,y2).dessineFleche(g);
		}
		if(res!=null)
		{
			String s = res.toString();
			if(s.length()>6)
				s=s.substring(0,6);
			g.drawString(s,(M1.x+M2.x)/2,(M1.y+M2.y)/2);
		}
	}
	
	public static double radToDeg(double r)
	{
		return (180 * r) / pi;
	}
	
	
	public String toString()
	{
		return "Transition " + eDepart.getnum() + "->" + eArrivee.getnum();
	}
	
}
		